@@ -1,24 +1,24 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
+
+from decimal import Decimal
+
 from trytond.i18n import gettext
-from trytond.pool import PoolMeta, Pool
 from trytond.model import ModelSQL, ModelView, fields
-
 from trytond.modules.product import round_price
+from trytond.pool import Pool, PoolMeta
 
+from .common import (
+    AmendmentLineMixin, get_moves, get_shipments_returns,
+    handle_shipment_exception_mixin, order_line_component_mixin,
+    order_line_mixin, order_mixin, search_moves, search_shipments_returns)
 from .product import ComponentMixin
-from .common import order_mixin, order_line_mixin, order_line_component_mixin
-from .common import get_shipments_returns, search_shipments_returns
-from .common import handle_shipment_exception_mixin
-from .common import AmendmentLineMixin
 
 
 class Purchase(order_mixin('purchase'), metaclass=PoolMeta):
     __name__ = 'purchase.purchase'
 
     def create_move(self, move_type):
-        pool = Pool()
-        Move = pool.get('stock.move')
         moves = super().create_move(move_type)
         for line in self.lines:
             if line.components:
@@ -26,7 +26,6 @@ class Purchase(order_mixin('purchase'), metaclass=PoolMeta):
                     move = component.get_move(move_type)
                     if move:
                         moves.append(move)
-        Move.save(moves)
         return moves
 
     @get_shipments_returns('stock.shipment.in')
@@ -47,6 +46,15 @@ class Purchase(order_mixin('purchase'), metaclass=PoolMeta):
     def search_shipment_returns(cls, name, clause):
         return super().search_shipment_returns(name, clause)
 
+    @get_moves
+    def get_moves(self, name):
+        return super().get_moves(name)
+
+    @classmethod
+    @search_moves
+    def search_moves(cls, name, clause):
+        return super().search_moves(name, clause)
+
 
 class Line(order_line_mixin('purchase'), metaclass=PoolMeta):
     __name__ = 'purchase.line'
@@ -54,7 +62,16 @@ class Line(order_line_mixin('purchase'), metaclass=PoolMeta):
     def get_component_order_line(self, component, **values):
         values = values.copy()
         values['purchase'] = self.purchase
-        return super().get_component_order_line(component, **values)
+        line = super().get_component_order_line(component, **values)
+        if line.unit_price is None:
+            line.unit_price = round_price(Decimal(0))
+        return line
+
+    @classmethod
+    def get_move_product_types(cls):
+        types = super().get_move_product_types()
+        types.append('kit')
+        return types
 
 
 class LineComponent(
@@ -87,7 +104,7 @@ class LineComponent(
 
         move = Move()
         move.quantity = quantity
-        move.uom = self.unit
+        move.unit = self.unit
         move.product = self.product
         move.from_location = self.line.from_location
         move.to_location = self.line.to_location
@@ -99,21 +116,44 @@ class LineComponent(
             move.currency = self.line.purchase.currency
         move.planned_date = self.line.planned_delivery_date
         move.origin = self
+        move.origin_planned_date = move.planned_date
         return move
 
     def _get_move_quantity(self, shipment_type):
         'Return the quantity that should be shipped'
         return abs(self.quantity)
 
+    def check_move_quantity(self):
+        from trytond.modules.purchase.exceptions import PurchaseMoveQuantity
+        pool = Pool()
+        Lang = pool.get('ir.lang')
+        Warning = pool.get('res.user.warning')
+        lang = Lang.get()
+        move_type = 'in' if self.quantity >= 0 else 'return'
+        quantity = (
+            self._get_move_quantity(move_type)
+            - self._get_shipped_quantity(move_type))
+        if quantity < 0:
+            warning_name = Warning.format(
+                'check_move_quantity', [self])
+            if Warning.check(warning_name):
+                raise PurchaseMoveQuantity(warning_name, gettext(
+                        'purchase.msg_purchase_line_move_quantity',
+                        line=self.rec_name,
+                        extra=lang.format_number_symbol(
+                            -quantity, self.unit),
+                        quantity=lang.format_number_symbol(
+                            self.quantity, self.unit)))
+
 
 class LineComponentIgnoredMove(ModelSQL):
     'Purchase Line Component - Ignored Move'
     __name__ = 'purchase.line.component-ignored-stock.move'
     component = fields.Many2One(
         'purchase.line.component', "Component",
-        ondelete='CASCADE', select=True, required=True)
+        ondelete='CASCADE', required=True)
     move = fields.Many2One(
-        'stock.move', "Move", ondelete='RESTRICT', select=True, required=True)
+        'stock.move', "Move", ondelete='RESTRICT', required=True)
 
 
 class LineComponentRecreatedMove(ModelSQL):
@@ -121,9 +161,9 @@ class LineComponentRecreatedMove(ModelSQL):
     __name__ = 'purchase.line.component-recreated-stock.move'
     component = fields.Many2One(
         'purchase.line.component', "Component",
-        ondelete='CASCADE', select=True, required=True)
+        ondelete='CASCADE', required=True)
     move = fields.Many2One(
-        'stock.move', "Move", ondelete='RESTRICT', select=True, required=True)
+        'stock.move', "Move", ondelete='RESTRICT', required=True)
 
 
 class HandleShipmentException(
