@@ -1,16 +1,15 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 from trytond.i18n import gettext
-from trytond.pool import PoolMeta, Pool
 from trytond.model import ModelSQL, ModelView, fields
-
 from trytond.modules.product import round_price
+from trytond.pool import Pool, PoolMeta
 
+from .common import (
+    AmendmentLineMixin, get_moves, get_shipments_returns,
+    handle_shipment_exception_mixin, order_line_component_mixin,
+    order_line_mixin, order_mixin, search_moves, search_shipments_returns)
 from .product import ComponentMixin
-from .common import order_mixin, order_line_mixin, order_line_component_mixin
-from .common import get_shipments_returns, search_shipments_returns
-from .common import handle_shipment_exception_mixin
-from .common import AmendmentLineMixin
 
 
 class Sale(order_mixin('sale'), metaclass=PoolMeta):
@@ -44,6 +43,15 @@ class Sale(order_mixin('sale'), metaclass=PoolMeta):
     def search_shipment_returns(cls, name, clause):
         return super().search_shipment_returns(name, clause)
 
+    @get_moves
+    def get_moves(self, name):
+        return super().get_moves(name)
+
+    @classmethod
+    @search_moves
+    def search_moves(cls, name, clause):
+        return super().search_moves(name, clause)
+
 
 class Line(order_line_mixin('sale'), metaclass=PoolMeta):
     __name__ = 'sale.line'
@@ -53,6 +61,12 @@ class Line(order_line_mixin('sale'), metaclass=PoolMeta):
         values['sale'] = self.sale
         return super().get_component_order_line(component, **values)
 
+    @classmethod
+    def get_move_product_types(cls):
+        types = super().get_move_product_types()
+        types.append('kit')
+        return types
+
 
 class LineComponent(
         order_line_component_mixin('sale'), ComponentMixin,
@@ -87,7 +101,7 @@ class LineComponent(
 
         move = Move()
         move.quantity = quantity
-        move.uom = self.unit
+        move.unit = self.unit
         move.product = self.product
         move.from_location = self.line.from_location
         move.to_location = self.line.to_location
@@ -99,6 +113,7 @@ class LineComponent(
             move.currency = self.line.sale.currency
         move.planned_date = self.line.planned_shipping_date
         move.origin = self
+        move.origin_planned_date = move.planned_date
         return move
 
     def _get_move_quantity(self, shipment_type):
@@ -117,15 +132,37 @@ class LineComponent(
                         invoice_line.quantity, self.line.unit)
             return self.quantity * quantity / self.line.quantity
 
+    def check_move_quantity(self):
+        from trytond.modules.sale.exceptions import SaleMoveQuantity
+        pool = Pool()
+        Lang = pool.get('ir.lang')
+        Warning = pool.get('res.user.warning')
+        lang = Lang.get()
+        move_type = 'in' if self.quantity >= 0 else 'return'
+        quantity = (
+            self._get_move_quantity(move_type)
+            - self._get_shipped_quantity(move_type))
+        if quantity < 0:
+            warning_name = Warning.format(
+                'check_move_quantity', [self])
+            if Warning.check(warning_name):
+                raise SaleMoveQuantity(warning_name, gettext(
+                        'sale.msg_sale_line_move_quantity',
+                        line=self.rec_name,
+                        extra=lang.format_number_symbol(
+                            -quantity, self.unit),
+                        quantity=lang.format_number_symbol(
+                            self.quantity, self.unit)))
+
 
 class LineComponentIgnoredMove(ModelSQL):
     'Sale Line Component - Ignored Move'
     __name__ = 'sale.line.component-ignored-stock.move'
     component = fields.Many2One(
         'sale.line.component', "Component",
-        ondelete='CASCADE', select=True, required=True)
+        ondelete='CASCADE', required=True)
     move = fields.Many2One(
-        'stock.move', "Move", ondelete='RESTRICT', select=True, required=True)
+        'stock.move', "Move", ondelete='RESTRICT', required=True)
 
 
 class LineComponentRecreatedMove(ModelSQL):
@@ -133,9 +170,9 @@ class LineComponentRecreatedMove(ModelSQL):
     __name__ = 'sale.line.component-recreated-stock.move'
     component = fields.Many2One(
         'sale.line.component', "Sale Line Component",
-        ondelete='CASCADE', select=True, required=True)
+        ondelete='CASCADE', required=True)
     move = fields.Many2One(
-        'stock.move', "Move", ondelete='RESTRICT', select=True, required=True)
+        'stock.move', "Move", ondelete='RESTRICT', required=True)
 
 
 class HandleShipmentException(
